/* --COPYRIGHT--,BSD
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  motor.c - Motor drive function functions
 *
 ******************************************************************************/

#include "motor.h"
#include "init.h"
#include "global.h"

// Speed close loop control function
void SpeedCloseLoop()
{
	// ADC start for A0
	ADC10CTL0 &= ~ENC;     				// Disable ADC10
	ADC10CTL1 = INCH_0;    				// Choose channel A0
	ADC10CTL0 |= ENC + ADC10SC;         // Sampling and conversion start
	while (ADC10CTL1 & ADC10BUSY);      // Hold while ADC10BUSY?
	adcResult = ADC10MEM; 				// Get the A0 ADC value 0 to 1024
	// Convert the ADC result into speed command
	speedCommand = ( ( (long)(SPEED_COMMAND_MAX-SPEED_COMMAND_MIN) \
					*adcResult)>>10 ) + SPEED_COMMAND_MIN;
	commandADC = (speedCommand>>2);
	// Start close loop control
	// Scale the calculation to 4 times
	fanSpeed_4times = MAIN_CLK / (speedCounter>>2);
	if (speedControlFlag == SPEED_CONTROL_ON)
	{
		// Feedback and "P" control
		speedControlOut = speedControlOut + speedCommand - fanSpeed_4times;
		// limit the MAX and MIN PWM output
		// Avoid UVLO of DRV10983
		if (speedControlOut >= PWM_MAX)
			speedControlOut = PWM_MAX;
		else if (speedControlOut <= PWM_MIN)
			speedControlOut = PWM_MIN;
		speedControlOutPWM = speedControlOut;
		speedControlFlag = SPEED_CONTROL_DONE;
	}
}

// Stop the stepper motor
// All the outputs of DRV8848 are HiZ
void StopStepper()
{
	AIN1_LOW;
	AIN2_LOW;
	BIN1_LOW;
	BIN2_LOW;
}

// Stepper motor control logic
void StepperLogic()
{
	if(microSteppingMode == MICRO_STEPPING_HALF)
	{
		// Half stepping logic
		switch(stepperIndex)
		{
		case STEPPER_INDEX_0: // 0 degree
			AIN1_HIGH;
			AIN2_LOW;
			BIN1_HIGH;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_45:STEPPER_INDEX_315;
			break;
		case STEPPER_INDEX_45: // 45 degree
			AIN1_HIGH;
			AIN2_LOW;
			BIN1_HIGH;
			BIN2_LOW;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_90:STEPPER_INDEX_0;
			break;
		case STEPPER_INDEX_90: // 90 degree
			AIN1_HIGH;
			AIN2_HIGH;
			BIN1_HIGH;
			BIN2_LOW;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_135:STEPPER_INDEX_45;
			break;
		case STEPPER_INDEX_135: // 135 degree
			AIN1_LOW;
			AIN2_HIGH;
			BIN1_HIGH;
			BIN2_LOW;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_180:STEPPER_INDEX_90;
			break;
		case STEPPER_INDEX_180: // 180 degree
			AIN1_LOW;
			AIN2_HIGH;
			BIN1_HIGH;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_225:STEPPER_INDEX_135;
			break;
		case STEPPER_INDEX_225: // 225 degree
			AIN1_LOW;
			AIN2_HIGH;
			BIN1_LOW;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_270:STEPPER_INDEX_180;
			break;
		case STEPPER_INDEX_270: // 270 degree
			AIN1_HIGH;
			AIN2_HIGH;
			BIN1_LOW;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_315:STEPPER_INDEX_225;
			break;
		case STEPPER_INDEX_315: // 315 degree
			AIN1_HIGH;
			AIN2_LOW;
			BIN1_LOW;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_0:STEPPER_INDEX_270;
			break;
		default:
			stepperIndex = STEPPER_INDEX_0; // Reset index when in abnormal
			break;
		}
		throttleSteps++; // Index counter update
	}
	else if(microSteppingMode == MICRO_STEPPING_FULL)
	{
		// Full step logic
		switch(stepperIndex)
		{
		case STEPPER_INDEX_0:
		case STEPPER_INDEX_45: // 0 degree
			AIN1_HIGH;
			AIN2_LOW;
			BIN1_HIGH;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_90:STEPPER_INDEX_270;
			break;
		case STEPPER_INDEX_90:
		case STEPPER_INDEX_135: // 90 degree
			AIN1_HIGH;
			AIN2_HIGH;
			BIN1_HIGH;
			BIN2_LOW;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_180:STEPPER_INDEX_0;
			break;
		case STEPPER_INDEX_180: // 180 degree
		case STEPPER_INDEX_225:
			AIN1_LOW;
			AIN2_HIGH;
			BIN1_HIGH;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_270:STEPPER_INDEX_90;
			break;
		case STEPPER_INDEX_270: // 279 degree
		case STEPPER_INDEX_315:
			AIN1_HIGH;
			AIN2_HIGH;
			BIN1_LOW;
			BIN2_HIGH;
			// Direction control
			stepperIndex = stepperDir?STEPPER_INDEX_0:STEPPER_INDEX_180;
			break;
		default:
			break;
		}
		throttleSteps++; // Index counter update
	}
}
